<?php

namespace GopNext\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;

class CalculationBudgetController {
    
    protected $crudService;
    protected $service;
  
    public function __construct($connection, $environment, $util, $crudService, $service) {
        $this->connection   = $connection;
        $this->environment  = $environment;
        $this->util         = $util;
        $this->crudService	= $crudService;
        $this->service      = $service;
    }
    
    public function budgetParameters(Request\Filter $request, Response $response) {
        $params = ['CDOPERADOR', 'NRORG'];
        $conditions = $this->util->prepareConditions($params);
        
        $data = $this->crudService->getFilteredDataSet('budgetParameters', $conditions)->getRows();
		$response->addDataSet(new DataSet("budgetParameters", $data));
    }
    
    public function gridSummary(Request\Filter $request, Response $response) {
        $params = ['CDOPERADOR', 'NRORG'];
        $conditions = $this->util->prepareConditions($params);
        $filter = $request->getFilterCriteria()->getConditions();
        
        foreach ($filter as $key => &$item) {
            if ($item["columnName"] === "NRPERIODOORCA" && is_array($item["value"])) {
                $item["value"] = $item["value"]["NRPERIODOORCA"];
            }
            if ($item["columnName"] === "IDAPROVACAOGOP" && $item["value"] === NULL) {
                unset($filter[$key]);
            }
        }
        
        $data = $this->crudService->getFilteredDataSet('gridSummary', array_merge($conditions, $filter))->getRows();
		$response->addDataSet(new DataSet("gridSummary", $data));
    }
    
    public function gridMainGrid(Request\Filter $request, Response $response) {
        $params = ['CDOPERADOR', 'NRORG'];
        $conditions = $this->util->prepareConditions($params);
        $filter = $request->getFilterCriteria()->getConditions();
        
        foreach ($filter as $key => &$item) {
            if ($item["columnName"] === "NRPERIODOORCA" && is_array($item["value"])) {
                $item["value"] = $item["value"]["NRPERIODOORCA"];
            }
            if ($item["columnName"] === "IDAPROVACAOGOP" && $item["value"] === NULL) {
                unset($filter[$key]);
            }
        }
        $data = $this->crudService->getFilteredDataSet('gop10000_mainGrid', array_merge($conditions, $filter))->getRows();
		$response->addDataSet(new DataSet("gop10000_mainGrid", $data));
    }
    
    public function changeBudgetMonth(Request\DataSet $request, Response $response) {
        $data = $request->getParameters();
        try {
            $this->service->beginTransaction();
            foreach ($data["dataset"] as $row) {
                $params = [
                    "NRORCAMENTOGOP" => $row["NRORCAMENTOGOP"],
                    "IDTIPOORCAOP"   => $row["IDTIPOORCAOP"],
                    "DTMESBASEITEM"  => $row["DTMESBASEITEM"],
                    "IDEDITMES"      => $row["IDEDITMES"],
                    "JANEIRO"        => $row["JANEIRO"],
                    "FEVEREIRO"      => $row["FEVEREIRO"],
                    "MARCO"          => $row["MARCO"],
                    "ABRIL"          => $row["ABRIL"],
                    "MAIO"           => $row["MAIO"],
                    "JUNHO"          => $row["JUNHO"],
                    "JULHO"          => $row["JULHO"],
                    "AGOSTO"         => $row["AGOSTO"],
                    "SETEMBRO"       => $row["SETEMBRO"],
                    "OUTUBRO"        => $row["OUTUBRO"],
                    "NOVEMBRO"       => $row["NOVEMBRO"],
                    "DEZEMBRO"       => $row["DEZEMBRO"],
                    "NRORG"          => $row["NRORG"]
                ];
                $this->service->changeBudgetMonth($params);
            }
          $this->service->commit();  
        }catch(\Exception $ex) {
          $this->service->rollback();
          $response->setError(new Error($ex->getMessage(), 500));
        }
    }
    
    public function budgetCalculation(Request\DataSet $request, Response $response) {
        $data = $request->getParameters();
        
        try {
            $this->service->beginTransaction();
            $nrLista = $this->util->insertLista($data['CDFILIAL']);
       		$this->service->budgetCalculation($data, $nrLista);
			$this->util->deleteLista($nrLista);
			
            $this->service->commit();
        } catch(\Exception $ex) {
            $this->service->rollback();
            $response->setError(new Error($ex->getMessage(), 500));
        }
    }
    
    public function workingDays(Request\Row $request, Response $response) {
        $parameters = $request->getRow();
        $changedValue = $parameters['EDITEDVALUE'];
        $params = ['NRORG'];
        $conditions = $this->util->prepareConditions($params);
        $dailyValue = 0;
        
        array_push($conditions, 
            array(
                "columnName" => "CDFILIAL",
                "operator"   => '=',
                "value"      => $parameters['CDFILIAL']
            ),
            array(
                "columnName" => "MES",
                "operator"   => '=',
                "value"      => $parameters['MES']
            ),
            array(
                "columnName" => "ANO",
                "operator"   => '=',
                "value"      => $parameters['ANO']
            )
        );
        $data = $this->crudService->getFilteredDataSet('workingDays', $conditions)->getRows();
        
        foreach ($data as $item) {
            if (intval($parameters['MES']) === intval(substr($item['MES'], -2))) {
                $dailyValue = $changedValue / $item['NRDIASUTEIS'];
            } else {
                $dataset['dataset'][] = array(
                    'MES' => intval(substr($item['MES'], -2)),
                    'VALOR' => $item['NRDIASUTEIS'] * $dailyValue,
                );
            }
        }
        
		$response->addDataSet(new DataSet("workingDays", $dataset));
    }
    
    public function gridDetail(Request\Filter $request, Response $response) {
        $params = ['NRORG'];
        $conditions = $this->util->prepareConditions($params);
        $filter = $request->getFilterCriteria()->getConditions();
        
        foreach ($filter as $key => &$item) {
            if ($item["columnName"] === "NRPERIODOORCA" && is_array($item["value"])) {
                $item["value"] = $item["value"]["NRPERIODOORCA"];
            }
            if ($item["columnName"] === "IDAPROVACAOGOP" && $item["value"] === NULL) {
                unset($filter[$key]);
            }
        }
        $data = $this->crudService->getFilteredDataSet('gop10000_gridDetail', array_merge($conditions, $filter))->getRows();
		$response->addDataSet(new DataSet("gop10000_gridDetail", $data));
    }
    
    public function changeBudgetDetailMonth(Request\DataSet $request, Response $response) {
        $data = $request->getParameters();
        $month = null;
        try {
            // var_dump($data["dataset"]);
            $this->service->beginTransaction();
            foreach ($data["dataset"] as $row) {
                $month = $row["NRITORCAMESGOP"];
                
                $params = [
                    "NRITDETALORCAMESGOP" => $row["NRITDETALORCAMESGOP"],
                    "VRORCAMENTO"         => isset($row["VRORCAMENTO"]) ? $row["VRORCAMENTO"] : null,
                    "QTORCAMENTO"         => isset($row["QTORCAMENTO"]) ? $row["QTORCAMENTO"] : null,
                    "VRPRECOBASE"         => isset($row["VRPRECOBASE"]) ? $row["VRPRECOBASE"] : null,
                    "NRORG"               => $this->environment->getNrOrgTrab()
                ];
                $this->service->changeBudgetDetailMonth($params);
            }
            if ($month) {
                $params = [
                    "NRITORCAMESGOP" => $month,
                    "NRORG"          => $this->environment->getNrOrgTrab()
                ];
                
                $this->service->updateTotalMonth($params);
            }
            
            $this->service->commit();  
        } catch(\Exception $ex) {
            $this->service->rollback();
            $response->setError(new Error($ex->getMessage(), 500));
        }
    }
    
    public function daysService(Request\Filter $request, Response $response) {
        $filter = $request->getFilterCriteria()->getConditions();
        $params = ['NRORG'];
        $conditions = $this->util->prepareConditions($params);
        
        $data = $this->crudService->getFilteredDataSet('daysService', array_merge($conditions, $filter))->getRows();
		$response->addDataSet(new DataSet("daysService", $data));
    }
    
    public function cleanDetailing(Request\DataSet $request, Response $response) {
        $data = $request->getParameters();
        
        try {
            $this->service->beginTransaction();
       		$this->service->cleanDetailing($data);
			
            $this->service->commit();
        } catch(\Exception $ex) {
            $this->service->rollback();
            $response->setError(new Error($ex->getMessage(), 500));
        }
    }
    
    public function changeBudgetDetMonthTotal(Request\DataSet $request, Response $response) {
        $data = $request->getParameters();
        try {
            $this->service->beginTransaction();
            $params = [
                "NRITORCAMESGOP" => $data['NRITORCAMESGOP'],
                "VRORCAMENTO"    => $data['VRORCAMENTO'],
                "NRORG"          => $this->environment->getNrOrgTrab()
            ];
            
            $this->service->changeBudgetDetMonthTotal($params);
            
            $this->service->commit();  
        } catch(\Exception $ex) {
            $this->service->rollback();
            $response->setError(new Error($ex->getMessage(), 500));
        }
    }
    
    public function serviceDetail(Request\Filter $request, Response $response) {
        $params = ['NRORG'];
        $conditions = $this->util->prepareConditions($params);
        $filter = $request->getFilterCriteria()->getConditions();
        $dataDiet = [];
        
        $data = $this->crudService->getFilteredDataSet('gop10000_serviceDetails', array_merge($conditions, $filter))->getRows();
        
        if (isset($data[0])) {
            $conditions = [
                array(
                    "columnName"   => 'CDFILIAL',
                    "value" => $data[0]['CDFILIAL'],
                    "operator" => "="
                ),
                array(
                    "columnName"   => 'CDSERVFILI',
                    "value" => $data[0]['CDSERVFILI'],
                    "operator" => "="
                )
            ];
            
            $dataDiet = $this->crudService->getFilteredDataSet('gop10000_serviceDetailsDiet', array_merge($conditions, $filter))->getRows();
        }
		$response->addDataSet(new DataSet("gop10000_serviceDetails", $data));
		$response->addDataSet(new DataSet("gop10000_serviceDetailsDiet", $dataDiet));
    }
    
    public function gop10000_importService(Request\DataSet $request, Response $response) {
        $data = $request->getParameters();
        
        try {
            $this->service->beginTransaction();
            
            $params = [
                "NRITDETALORCAMESGOP" => $data['NRITDETALORCAMESGOP']
            ];
            if ($data['IMPORT'] === 'Cust') {
                $this->service->gop10000_importServiceCust($params);
            } else {
                $this->service->gop10000_importServiceDiners($params);
            }
            
            $this->service->commit();
        } catch(\Exception $ex) {
            $this->service->rollback();
            $response->setError(new Error($ex->getMessage(), 500));
        }
    }
    
    public function gop10000_cleanService(Request\DataSet $request, Response $response) {
        $data = $request->getParameters();
        
        try {
            $this->service->beginTransaction();
       		$this->service->gop10000_cleanService($data);
			
            $this->service->commit();
        } catch(\Exception $ex) {
            $this->service->rollback();
            $response->setError(new Error($ex->getMessage(), 500));
        }
    }
    
    public function gop10000_saveChangeServiceDetail(Request\DataSet $request, Response $response) {
        $filters = $request->getParameters()["dataset"]["filter"];
        
        $nritDetalOrcaMesGOP = null;
        $dietPad = null;
        $dataValues = [];
        
        foreach ($filters as $filter) {
            if ($filter['name'] === 'NRITDETALORCAMESGOP') {
                $nritDetalOrcaMesGOP = $filter['value'];
            } elseif ($filter['name'] === 'CDDIETPAD') {
                $dietPad = $filter['value'];
            } elseif ($filter['name'] === 'DATA') {
                $dataValues = $filter['value'];
            }
        }
        
        try {
            $this->service->beginTransaction();
            
            $params = [
                "NRITDETALORCAMESGOP" => $nritDetalOrcaMesGOP,
                "CDDIETPAD"           => $dietPad,
                "VRCUSTPADR"          => $dataValues["VRCUSTPADR"],
                "QTSEG"               => $dataValues["QTSEG"],
                "QTTER"               => $dataValues["QTTER"],
                "QTQUA"               => $dataValues["QTQUA"],
                "QTQUI"               => $dataValues["QTQUI"],
                "QTSEX"               => $dataValues["QTSEX"],
                "QTSAB"               => $dataValues["QTSAB"],
                "QTDOM"               => $dataValues["QTDOM"]
            ];
            
            $this->service->gop10000_saveChangeServiceDetail($params);
            
            $this->service->commit();  
        }catch(\Exception $ex) {
            $this->service->rollback();
            $response->setError(new Error($ex->getMessage(), 500));
        }
    }
    
    public function repeatBudget(Request\DataSet $request, Response $response) {
        $data = $request->getParameters();
        
        try {
            $this->service->beginTransaction();
       		$this->service->repeatBudget($data);
			
            $this->service->commit();
        } catch(\Exception $ex) {
            $this->service->rollback();
            $response->setError(new Error($ex->getMessage(), 500));
        }
    }
    
    public function configBudgetAdjustment(Request\Filter $request, Response $response) {
        $filter = $request->getFilterCriteria()->getConditions();
        
        try {
            $this->service->beginTransaction();
            $nrLista = $this->util->insertLista($filter[0]['value']);
            $filter[0]['value'] = $nrLista;
        
            $dataMonths = $this->crudService->getFilteredDataSet('budgetMonths', [])->getRows();
            // var_dump($filter);
            $dataDetail = $this->crudService->getFilteredDataSet('detailBudget', $filter)->getRows();
            
            $this->util->deleteLista($nrLista);
        
            $this->service->commit();
            
    		$response->addDataSet(new DataSet("budgetMonths", $dataMonths));
    		$response->addDataSet(new DataSet("detailBudget", $dataDetail));
        } catch(\Exception $ex) {
            $this->service->rollback();
            $response->setError(new Error($ex->getMessage(), 500));
        }
    }
    
    public function adjustmentsBudget(Request\DataSet $request, Response $response) {
        $data = $request->getParameters();
        $nrListaItem = null;
        
        try {
            $this->service->beginTransaction();
            $nrListaUnit = $this->util->insertLista($data['CDFILIAL']);
            $months = '_' . implode('_', $data['MONTH']) . '_';
            if (!empty($data['ITEM'])) {
                $nrListaItem = $this->util->insertLista($data['ITEM']);
            }
       		$this->service->adjustmentsBudget($data, $nrListaUnit, $nrListaItem, $months);
			$this->util->deleteLista($nrListaUnit);
			
            $this->service->commit();
        } catch(\Exception $ex) {
            $this->service->rollback();
            $response->setError(new Error($ex->getMessage(), 500));
        }
    }
}
