import Vue from 'vue';
import { ZdFieldGroup } from '@zeedhi/zd-field-group';
import {
  Http,
  Metadata,
  IEventParam,
  Loader,
  I18n,
} from '@zeedhi/core';
import {
  SelectMultiple,
  Select,
  Form,
  Text,
  Checkbox,
  Grid,
  LoadingService,
  AlertService,
} from '@zeedhi/common';

Vue.component('ZdFieldGroup', ZdFieldGroup);

export class OperatingBudgetParametersController {
  public showSaveSuccessAlert() {
    AlertService.show({
      name: 'saveSucessAlert',
      text: I18n.translate('Parametrização salva com sucesso.'),
      color: 'success',
      timeout: 3000,
      top: true,
      right: true,
    });
  }
  
  public showSaveFailedAlert() {
    AlertService.show({
      name: 'saveFailedAlert',
      text: I18n.translate('Verifique os campos inválidos.'),
      color: 'error',
      timeout: 5000,
      top: true,
      right: true,
    });
  }

  public buttonStateGeneral = true;
  
  public activateButtonGeneral() {
    this.buttonStateGeneral = false;
  }
  
  public buttonStateUnit = true;
  
  public activateButtonUnit() {
    this.buttonStateUnit = false;
    this.previewCheckbox();
  }
  
  public createModals() {
    const registeProfile = Loader.getInstance('RegisterOperatingParamsModalController');
    registeProfile.createRegisterOperatingParamsModal();
  }

  public checkInvalue() {
    const formBudgetCycle = Metadata.getInstance<Form>('budgetCycleForm');
    const checkboxValue = formBudgetCycle.getChildInstance<Checkbox>('IDUTILORCAREALOC').value;
    if (checkboxValue === 'S') {
      Metadata.getInstance<Select>('IDREALSUPORCA').addValidation('required');
      Metadata.getInstance<Select>('IDREALINFORCA').addValidation('required');
    } else {
      Metadata.getInstance<Select>('IDREALSUPORCA').clearValidations();
      Metadata.getInstance<Select>('IDREALINFORCA').clearValidations();
    }
    const fieldsRequired = checkboxValue === 'S';

    formBudgetCycle.getChildInstance<Checkbox>('IDCONSORCREALOC').disabled = !fieldsRequired;
    formBudgetCycle.getChildInstance<Select>('IDREALSUPORCA').disabled = !fieldsRequired;
    formBudgetCycle.getChildInstance<Select>('IDREALINFORCA').disabled = !fieldsRequired;

    if (fieldsRequired) {
      formBudgetCycle.validate();
    }
  }

  public validateGeneralFields() {
    const arrayValue = [];
    const operationalItem = Metadata.getInstance<SelectMultiple>('OperationalitemsSelect');

    Http.post('/operationalItem', {
      requestType: 'DataSet',
      dataset: operationalItem.datasource.data,
    }).then((response) => {
      const data = response.data.dataset.validateOperationalitemsSelect;
      this.reciveBudgetOperation(data[1][0]);
      data[0].forEach((row) => {
        if (row.IDCHECK === 'S') {
          arrayValue.push(row.NRITEMOPERACIONAL, row.IDCHECK);
        }
      });
      operationalItem.setValue(arrayValue);
    });
  }
  
  public reciveBudgetOperation(data) {
    const formIndexContractCorrection = Metadata.getInstance<Form>('indexContractCorrectionForm');
    const formCalculationConsumption = Metadata.getInstance<Form>('calculationConsumptionForm');
    const formResultSimulation = Metadata.getInstance<Form>('resultSimulationBasedForm');
    const formBudgetDetailing = Metadata.getInstance<Form>('budgetDetailForm');
    const formProfile = Metadata.getInstance<Form>('formBillingCalculation');
    const formBudgetCycle = Metadata.getInstance<Form>('budgetCycleForm');
    
    formProfile.value.IDPERORCAITEMDETAL = data.IDPERORCAITEMDETAL;
    formProfile.value.IDUTILVERSAOORCA = data.IDUTILVERSAOORCA;
    formProfile.value.IDAPROVORCACONC = data.IDAPROVORCACONC;
    formProfile.value.IDUTILMESBASE = data.IDUTILMESBASE;
    formProfile.value.IDCALCFATUROP = data.IDCALCFATUROP;
    formProfile.value.NRDIALIMITEORCA = data.NRDIALIMITEORCA;
    
    formBudgetDetailing.value.IDORCDESPDIVERS = data.IDORCDESPDIVERS;
    formBudgetDetailing.value.IDORCADESPMATRI = data.IDORCADESPMATRI;
    formBudgetDetailing.value.IDORCDESPENTROP = data.IDORCDESPENTROP;
    
    formBudgetCycle.value.NRDIAANTELABORCA = data.NRDIAANTELABORCA;
    formBudgetCycle.value.IDUTILORCAREALOC = data.IDUTILORCAREALOC;
    formBudgetCycle.value.IDCONSORCREALOC = data.IDCONSORCREALOC;
    formBudgetCycle.value.IDPERIODOORCA = data.IDPERIODOORCA;
    formBudgetCycle.value.IDREALINFORCA = data.IDREALINFORCA;
    formBudgetCycle.value.IDREALSUPORCA = data.IDREALSUPORCA;
    formBudgetCycle.value.NRDIAINIORCA = data.NRDIAINIORCA;
    formBudgetCycle.value.NRMESINIORCA = data.NRMESINIORCA;
    
    formResultSimulation.value.IDTIPOSIMULARESULTFAT = data.IDTIPOSIMULARESULTFAT;
    formIndexContractCorrection.value.CDINDICECON = data.CDINDICECON;
    formIndexContractCorrection.value.CDINDICEFAT = data.CDINDICEFAT;
    formCalculationConsumption.value.IDUTILMESBASE = data.IDUTILMESBASE;
    formCalculationConsumption.value.CDTIPOCUSTO = data.CDTIPOCUSTO;

    this.checkInvalue();
    this.buttonStateGeneral = true;
  }
  
  public saveOpBudgetParamsOperation() {
    const formCalculationConsumption = Metadata.getInstance<Form>('calculationConsumptionForm');
    const formResultSimulation = Metadata.getInstance<Form>('resultSimulationBasedForm');
    const formIndexContract = Metadata.getInstance<Form>('indexContractCorrectionForm');
    const formBudgetDetailing = Metadata.getInstance<Form>('budgetDetailForm');
    const formProfile = Metadata.getInstance<Form>('formBillingCalculation');
    const formBudgetCycle = Metadata.getInstance<Form>('budgetCycleForm');
    
    const sendObject = {
      IDTIPOSIMULARESULTFAT: formResultSimulation.value.IDTIPOSIMULARESULTFAT,
      CDTIPOCUSTO: formCalculationConsumption.value.CDTIPOCUSTO,
      CDINDICECON: formIndexContract.value.CDINDICECON,
      CDINDICEFAT: formIndexContract.value.CDINDICEFAT,

      IDORCDESPDIVERS: formBudgetDetailing.value.IDORCDESPDIVERS,
      IDORCADESPMATRI: formBudgetDetailing.value.IDORCADESPMATRI,
      IDORCDESPENTROP: formBudgetDetailing.value.IDORCDESPENTROP,

      IDPERORCAITEMDETAL: formProfile.value.IDPERORCAITEMDETAL,
      IDUTILVERSAOORCA: formProfile.value.IDUTILVERSAOORCA,
      IDAPROVORCACONC: formProfile.value.IDAPROVORCACONC,
      IDUTILMESBASE: formProfile.value.IDUTILMESBASE,
      IDCALCFATUROP: formProfile.value.IDCALCFATUROP,
      NRDIALIMITEORCA: formProfile.value.NRDIALIMITEORCA,

      NRDIAANTELABORCA: formBudgetCycle.value.NRDIAANTELABORCA,
      IDUTILORCAREALOC: formBudgetCycle.value.IDUTILORCAREALOC,
      IDCONSORCREALOC: formBudgetCycle.value.IDCONSORCREALOC,
      IDPERIODOORCA: formBudgetCycle.value.IDPERIODOORCA,
      IDREALINFORCA: formBudgetCycle.value.IDREALINFORCA,
      IDREALSUPORCA: formBudgetCycle.value.IDREALSUPORCA,
      NRDIAINIORCA: formBudgetCycle.value.NRDIAINIORCA,
      NRMESINIORCA: formBudgetCycle.value.NRMESINIORCA,

    };
    // switch (checkboxValue) {
    //   case 'N':
    //     Metadata.getInstance<Select>('IDREALSUPORCA').clearValidations();
    //     Metadata.getInstance<Select>('IDREALINFORCA').clearValidations();
    //     break;
    //   case 'S':
    //     Metadata.getInstance<Select>('IDREALSUPORCA').addValidation('required');
    //     Metadata.getInstance<Select>('IDREALINFORCA').addValidation('required');
    //     break;
    //   default:
    //     break;
    // }
    if (formBudgetCycle.validate() && formBudgetDetailing.validate()) {
      Http.post('/saveOpBudgetParamsOperation', {
        requestType: 'Row',
        row: sendObject,
      });
      this.checkInvalue();
      this.saveOpBudgetParamsItem();
      this.buttonStateGeneral = true;
      this.loadOperatingParameter();
      this.showSaveSuccessAlert();
    } else {
      this.showSaveFailedAlert();
    }
  }
  
  public populateOpBudgetParamsGrid() {
    const grid = Metadata.getInstance<Grid>('gridUnit');
    Http.get('/populateOpBudgetParamsGrid').then((response) => {
      const data = response.data.dataset.populateOpBudgetParamsGrid;
      grid.datasource.updateData(data);
    });
  }
  
  public populateOpBudgetParamsUnit(row) {
    const textUnit = Metadata.getInstance<Text>('textComplexUnitName');
    textUnit.text = `Unidade: ${row.CDFILIAL} - ${row.NMFILIAL}`;
    const sendObject = {
      CDFILIAL: row.CDFILIAL,
    };
    Http.post('/populateOpBudgetParamsUnit', {
      requestType: 'Row',
      row: sendObject,
    }).then((response) => {
      const data = response.data.dataset.operatingUnit;
      this.reciveFormUnit(data[0]);
    });
  }
  
  public reciveFormUnit(data) {
    const formUnit = Metadata.getInstance<Form>('formUnit');
    
    formUnit.value.NRMESREAJCONTR = data.NRMESREAJCONTR;
    formUnit.value.VRPEORCBLOQFIL = data.VRPEORCBLOQFIL;
    formUnit.value.VRPEORCALERTFIL = data.VRPEORCALERTFIL;
    formUnit.value.IDCALCOMESTORC = data.IDCALCOMESTORC;
    formUnit.value.IDCONSAPENCARDAPROV = data.IDCONSAPENCARDAPROV;
    formUnit.value.NRDIAINIORCAGOP = data.NRDIAINIORCAGOP;
    formUnit.value.IDUTILPERCFILGOP = data.IDUTILPERCFILGOP;
    
    this.buttonStateUnit = true;
    this.previewCheckbox();
  }
  
  public previewCheckbox() {
    const formCheckBox = Metadata.getInstance<Form>('formUnit');
    const checkboxValue = formCheckBox.getChildInstance<Checkbox>('IDCALCOMESTORC').value;

    formCheckBox.getChildInstance<Checkbox>('IDCONSAPENCARDAPROV').isVisible = checkboxValue !== 'S';
  }

  public form?: Form;

  public loadFormInstance() {
    this.form = Metadata.getInstance<Form>('formUnit');
  }

  public formVisible = false;

  public get visibleCardGrid() {
    return !this.formVisible;
  }

  public clickVisibleControll() {
    this.form!.clearForm();
    this.formVisible = false;
  }

  public async clickRowGrid({ row }: IEventParam<Grid>) {
    this.populateOpBudgetParamsUnit(row);
    this.formVisible = true;
    this.form!.value = {
      CDFILIAL: row.CDFILIAL,
      nameUnit: row.NMFILIAL,
    };
  }
  
  public saveOpBudgetParamsUnit() {
    const formUnit = Metadata.getInstance<Form>('formUnit');
    const sendObject = {
      NRMESREAJCONTR: formUnit.value.NRMESREAJCONTR,
      VRPEORCBLOQFIL: formUnit.value.VRPEORCBLOQFIL,
      VRPEORCALERTFIL: formUnit.value.VRPEORCALERTFIL,
      IDCALCOMESTORC: formUnit.value.IDCALCOMESTORC,
      IDCONSAPENCARDAPROV: formUnit.value.IDCONSAPENCARDAPROV,
      NRDIAINIORCAGOP: formUnit.value.NRDIAINIORCAGOP,
      IDUTILPERCFILGOP: formUnit.value.IDUTILPERCFILGOP,
      CDFILIAL: formUnit.value.CDFILIAL,
    };
    if (formUnit.validate()) {
      Http.post('/saveOpBudgetParamsUnit', {
        requestType: 'Row',
        row: sendObject,
      });
      this.showSaveSuccessAlert();
      this.buttonStateUnit = true;
      this.loadOperatingParameter();
    } else {
      this.showSaveFailedAlert();
    }
  }
  
  public cancelUnitButton() {
    const gridUnit = Metadata.getInstance<Grid>('gridUnit').datasource;
    const cdfilial = {
      CDFILIAL: gridUnit.currentRow.CDFILIAL,
      NMFILIAL: gridUnit.currentRow.NMFILIAL,
    };
    this.populateOpBudgetParamsUnit(cdfilial);
    this.loadOperatingParameter();
  }
  
  public saveOpBudgetParamsItem() {
    const formBudgetDetailing = Metadata.getInstance<Form>('budgetDetailForm');
    if (formBudgetDetailing.value.OperationalitemsSelect != null) {
      const sendObject = {
        NRITEMOPERACIONAL: formBudgetDetailing.value.OperationalitemsSelect,
      };
      Http.post('/saveOpBudgetParamsItem', {
        requestType: 'Row',
        row: sendObject,
      });
    } else {
      formBudgetDetailing.validate();
    }
  }
  
  public messageValue: string = 'Carregando';
  
  public loadOperatingParameter() {
    LoadingService.show(this.messageValue);
    window.setTimeout(() => LoadingService.hide(), 2500);
  }
  
  public cancelGeneralOperatingParams() {
    this.validateGeneralFields();
    this.loadOperatingParameter();
  }
}
