import {
  IEventParam,
  Metadata,
  Http,
} from '@zeedhi/core';
import {
  TekGrid,
} from '@zeedhi/teknisa-components-common';
import {
  LoadingService,
  Text,
  Select,
} from '@zeedhi/common';
import { Utilities } from '@/util/Utilities';

export class PercentageAndStructureForBudgetController {
  private employeeCurrentRow: any = {};
  
  public previewStructure = 1;
  
  public rowsEditable = [];
  
  private gridItemsInEditing = false;
  
  private gridDetailInEditing = false;
  
  public beforeCancelItems() {
    this.gridItemsInEditing = false;
  }
  
  public beforeCancelDetail() {
    this.gridDetailInEditing = false;
  }
  
  public beforeReloadDetail() {
    const routeName = 'detailsOperationalItemsStructure';
    const gridItems = Metadata.getInstance<TekGrid>('gridOperationalItems');
    const gridDetail = Metadata.getInstance<TekGrid>('detailsOperationalItems');
    const { currentRow } = gridItems.datasource;
    const sendObject = {
      filter: [{
        name: 'NRSEQITEMOP',
        operator: '=',
        value: currentRow.NRSEQ,
      },
      {
        name: 'CDFILIAL',
        operator: '=',
        value: currentRow.CDFILIAL,
      }],
    };
    LoadingService.show();
    Http.post(`/${routeName}`, sendObject).then((response) => {
      const data = response.data.dataset[routeName];
      gridDetail.datasource.setFilter([]);
      gridDetail.datasource.updateData(data);
      this.setSourceOptions(currentRow.NRSEQITEMORCA, currentRow.CDFILIAL);
      LoadingService.hide();
    }).catch((e) => {
      LoadingService.hide();
      console.warn(e);
    });
  }
  
  public checkGridItemsInEditing({ component }: IEventParam<TekGrid>) {
    this.gridItemsInEditing = component.editing;
  }
  
  public checkGridDetailInEditing({ component }: IEventParam<TekGrid>) {
    this.gridDetailInEditing = component.editing;
  }
  
  get disabledCardDetail() {
    return this.gridItemsInEditing;
  }
  
  get disabledCardItemsActionDetail() {
    return this.gridDetailInEditing;
  }
  
  get disabledCardUnit() {
    return this.gridItemsInEditing || this.gridDetailInEditing;
  }
  
  public structureItemChange() {
    const utilities = new Utilities();
    const gridItems = Metadata.getInstance<TekGrid>('gridOperationalItems');
    const data = gridItems.getEditedRows();
    LoadingService.show();
    Http.post('/structureItemChange', {
      requestType: 'DataSet',
      dataset: data,
    }).then(async () => {
      // await grid.saveEditedRows();
      // this.onInlineEditUpdateRow({ component: grid });
      utilities.successMessage();
    }).catch((e) => {
      utilities.errorMessage();
      console.warn(e);
    }).finally(() => {
      this.gridItemsInEditing = false;
      LoadingService.hide();
    });
  }
  
  public unitRowClick() {
    const gridUnits = Metadata.getInstance<TekGrid>('gridUnits');
    const selectedUnit = `${gridUnits.datasource.currentRow.CDFILIAL} - ${gridUnits.datasource.currentRow.NMFILIAL}`;
    const routeName = 'operationalItemStructure';
    const grid = Metadata.getInstance<TekGrid>('gridOperationalItems');
    const gridDetail = Metadata.getInstance<TekGrid>('detailsOperationalItems');
    const unit = Metadata.getInstance<Text>('selectedUnit');
    
    const sendObject = {
      filter: [{
        name: 'CDFILIAL',
        operator: '=',
        value: gridUnits.datasource.currentRow.CDFILIAL,
      }],
    };
    let data;
    
    LoadingService.show();
    Http.post(`/${routeName}`, sendObject).then((response) => {
      data = response.data.dataset[routeName];
      grid.datasource.setFilter([]);
      grid.datasource.updateData(data);
      gridDetail.datasource.setFilter([]);
      gridDetail.datasource.updateData([]);
      
      LoadingService.hide();
    }).catch((e) => {
      LoadingService.hide();
      console.warn(e);
    });
    
    unit.text = selectedUnit;
  }
  
  public visiblePercent(grid, currentRow) {
    const columns = grid.columns;
    const columnVrPercent = columns.filter((column) => (column.name === 'VRPERCENTUAL'));
    if (currentRow.NRSEQITEMORCA === 0) {
      (columnVrPercent as any)[0].isVisible = false;
      (columnVrPercent as any)[0].isVisibleValue = false;
    } else {
      (columnVrPercent as any)[0].isvisible = true;
      (columnVrPercent as any)[0].isVisibleValue = true;
    }
  }
  
  public rowOperationalItemClick({ component }: any) {
    const routeName = 'detailsOperationalItemsStructure';
    const grid = Metadata.getInstance<TekGrid>('detailsOperationalItems');
    const { currentRow } = component.datasource;
    if (currentRow.NRSEQITEMORCA === 11) {
      component.parent.height = '96%';
      grid.parent.isVisible = false;
    } else {
      const sendObject = {
        filter: [{
          name: 'NRSEQITEMOP',
          operator: '=',
          value: currentRow.NRSEQ,
        },
        {
          name: 'CDFILIAL',
          operator: '=',
          value: currentRow.CDFILIAL,
        }],
      };
      LoadingService.show();
      Http.post(`/${routeName}`, sendObject).then((response) => {
        const data = response.data.dataset[routeName];
        grid.datasource.setFilter([]);
        grid.datasource.updateData(data);
        this.setSourceOptions(currentRow.NRSEQITEMORCA, currentRow.CDFILIAL);
      }).catch((e) => {
        console.warn(e);
      }).finally(() => {
        component.parent.height = '30%';
        grid.parent.isVisible = true;
        this.visiblePercent(grid, currentRow);
        LoadingService.hide();
      });
    }
  }
  
  public setSourceOptions(nrSeqItemOrca, cdFilial) {
    const grid = Metadata.getInstance<TekGrid>('detailsOperationalItems');
    const serviceOption = { name: 'Serviço', value: 'SERVICO' };
    const productOption = { name: 'Produto', value: 'PRODUTO' };
    const subProductOption = { name: 'Subgrupo', value: 'SUBGRUPOPROD' };
    const billToPayOption = { name: 'Conta a Pagar', value: 'TCPAGAR' };
    const accountOption = { name: 'Conta Contábil', value: 'CONTABIL' };
    const tipoOpOption = { name: 'Tipo Operação', value: 'TIPOOPERAC' };
    const origin = [
      { NRSEQITEM: [0], DATA: [{ name: 'Diversos', value: 'DIVERSOS' }] },
      { NRSEQITEM: [1, 8, 9], DATA: [{ name: 'Imposto', value: 'IMPOSTO' }] },
      { NRSEQITEM: [2], DATA: [] },
      { NRSEQITEM: [3], DATA: [{ name: 'Mão de Obra', value: 'ORCAMENTOMO' }] },
      { NRSEQITEM: [5, 6, 7], DATA: [{ name: 'Despesas', value: 'DESPESA' }] },
      { NRSEQITEM: [10], DATA: [{ name: 'Investimentos', value: 'INVESTIMENTOS' }] },
      { NRSEQITEM: [11], DATA: [{ name: 'Depreciação', value: 'DEPRECIACAO' }] },
    ];
    const originItem = origin.filter((item) => (item.NRSEQITEM.indexOf(nrSeqItemOrca) !== -1));
    const data = originItem[0].DATA;
    switch (nrSeqItemOrca) {
      case 0:
      case 2:
        if (cdFilial !== 'P') {
          data.push(serviceOption);
        }
        data.push(productOption);
        data.push(subProductOption);
        break;
      case 1:
      case 10:
        data.push(billToPayOption);
        break;
      case 3:
        data.push(billToPayOption);
        data.push(accountOption);
        break;
      case 6:
        data.push(productOption);
        data.push(subProductOption);
        data.push(billToPayOption);
        data.push(accountOption);
        break;
      case 7:
        data.push(tipoOpOption);
        break;
      default:
        console.warn();
    }
    const columns = grid.columns;
    const columnOrigin = columns.filter((item) => (item.name === 'DSORIGEM'));
    columnOrigin[0].componentProps.datasource.data = data;
  }
  
  public setRouteDescription({ row, component }) {
    const gridUnits = Metadata.getInstance<TekGrid>('gridUnits');
    const gridItems = Metadata.getInstance<TekGrid>('gridOperationalItems');
    const originRow = component ? component.value : row.CDORIGEM;
    const cdFilial = gridUnits.datasource.currentRow.CDFILIAL;
    const nrSeqItem = gridItems.datasource.currentRow.NRSEQITEMORCA;
    const routeDescription = [
      // FATURAMENTO (FATURAMENTO DIVERSO)----- OK ------
      {
        NRSEQITEM: [0], ORIGIN: 'DIVERSOS', route: '/crudN/descriptionBilling', params: {},
      },
      // IMPOSTO , IMPOSTO A RECUPERAR, IMPOSTO SOBRE O LUCRO ------- OK ------
      {
        NRSEQITEM: [1, 8, 9], ORIGIN: 'IMPOSTO', route: '/crudN/descriptionTax', params: {},
      },
      // MAO DE OBRA ----- OK -----------
      {
        NRSEQITEM: [3], ORIGIN: 'ORCAMENTOMO', route: '/crudN/descriptionLabor', params: { CDFILIAL: cdFilial },
      },
      // DESPESAS DA MATRIZ ------ OK ------
      {
        NRSEQITEM: [5], ORIGIN: 'DESPESA', route: '/crudN/descriptionHeadOfficeExpenses', params: {},
      },
      // DESPESAS DIVERSAS ------ OK -------
      {
        NRSEQITEM: [6], ORIGIN: 'DESPESA', route: '/crudN/descriptionMiscellaneousExpenses', params: {},
      },
      // DESPESAS LANCAMENTO ENTRADA DO TIPO DESPESA ------- OK -------
      {
        NRSEQITEM: [7], ORIGIN: 'DESPESA', route: '/crudN/descriptionExpensesEntryReleaseDesp', params: {},
      },
      // DESPESAS LANCAMENTO ENTRADA DO TIPO OPERACAO ------ OK -----
      {
        NRSEQITEM: [7], ORIGIN: 'TIPOOPERAC', route: '/crudN/descriptionExpensesEntryReleaseOp', params: {},
      },
      // INVESTIMENTOS ----- OK -------
      {
        NRSEQITEM: [10], ORIGIN: 'INVESTIMENTOS', route: '/crudN/descriptionInvestment', params: {},
      },
      // SERVICO - FATURAMENTO OU CONSUMO ------- OK ------
      {
        NRSEQITEM: [0, 2], ORIGIN: 'SERVICO', route: '/crudN/descriptionService', params: { CDFILIAL: cdFilial },
      },
      // PRODUTO - FATURAMENTO, CONSUMO, DESPESAS DIVERSAS ----- OK -----
      {
        NRSEQITEM: [0, 2, 6], ORIGIN: 'PRODUTO', route: '/crudN/descriptionProduct', params: {},
      },
      // SUBGRUPO DE PRODUTO - FATURAMENTO, CONSUMO, DESPESA DIVERSA ----
      {
        NRSEQITEM: [0, 2, 6], ORIGIN: 'SUBGRUPOPROD', route: '/crudN/descriptionSubGroup', params: {},
      },
      // CONTA CONTABIL - DESPESAS DIVERSAS , LANC ENTRA, MAO OBRA ---- OK ----
      {
        NRSEQITEM: [3, 6, 7], ORIGIN: 'CONTABIL', route: '/crudN/descriptionAccount', params: {},
      },
      // CONTA A PAGAR - DESPESAS DIVERSAS , LANC ENTRA, MAO OBRA ---- OK ----
      {
        NRSEQITEM: [3, 6, 7], ORIGIN: 'TCPAGAR', route: '/crudN/descriptionBillsToPay', params: {},
      },
    ];
    const result = routeDescription.filter((item) => item.NRSEQITEM.indexOf(nrSeqItem) !== -1 && item.ORIGIN === originRow);
    const route = result.length > 0 ? result[0].route : '';
    const params = result.length > 0 ? result[0].params : {};
    if (component) {
      const gridDetail = Metadata.getInstance<TekGrid>('detailsOperationalItems');
      const fieldDescription = gridDetail.getComponent(row.NRSEQUENCIAL, 'DSDESCRICAO');
      (fieldDescription as any).datasource.updateData([]);
      (fieldDescription as any).datasource.route = route;
      (fieldDescription as any).datasource.filter = params;
      (fieldDescription as any).datasource.get();
      gridDetail.reapplyConditions(row);
    }
    return {
      route,
      uniqueKey: 'CDCODIGO',
      type: 'rest',
      limit: 10,
      lazyLoad: true,
      filter: params,
    };
  }
  
  public onChangePreviewStructure({ component }) {
    if (component.datasource.currentRowValue) {
      this.previewStructure = component.datasource.currentRowValue.value;
    }
  }
  
  public setPreviewStructure({ component }) {
    component.setValue(this.previewStructure);
  }
  
  get unitView() {
    return this.previewStructure === 1;
  }
  
  get detailedView() {
    return this.previewStructure === 2;
  }
  
  public cellIsEditableItem({ row }) {
    return row.NRSEQITEMORCA !== 0;
  }
  
  public budgetUnits({ component }: any) {
    const routeName = 'budgetUnits';
    const sendObject = {};
    
    LoadingService.show();
    Http.post(`/${routeName}`, sendObject).then((response) => {
      component.datasource.setFilter([]);
      component.datasource.updateData(response.data.dataset[routeName]);
        
      LoadingService.hide();
    }).catch((e) => {
      // utilities.errorMessage(I18n.translate('gop::FAILED_DATA'));
      LoadingService.hide();
      console.warn(e);
    });
  }
  
  public addNewRow({ component }: IEventParam<TekGrid>) {
    let newId = 0;
    component.datasource.data.forEach((obj) => {
      newId = obj.NRSEQUENCIAL > newId ? obj.NRSEQUENCIAL : newId;
    });
    newId += 1;
    if (this.rowsEditable.indexOf(newId) === -1) {
      this.rowsEditable.push(newId);
    }
    component.addNewRow({ NRSEQUENCIAL: newId }, 'start').then(() => {
      const nameInput = Metadata.getInstance<Select>(`DSORIGEM_editable_${newId}`);
      nameInput.setFocus();
    });
    this.gridDetailInEditing = true;
  }
  
  public editableOrigin({ row }) {
    const newRow = this.rowsEditable.indexOf(row.NRSEQUENCIAL) !== -1;
    return newRow;
  }
  
  public visibleActionDetail({ row }) {
    return row.CDORIGEM === 'DESPESA' || row.CDORIGEM === 'ORCAMENTOMO';
  }
  
  public setCode({ component, row }) {
    const gridDetail = Metadata.getInstance<TekGrid>('detailsOperationalItems');
    const newCodeDescription = component.selectValue.CDCODIGO;
    gridDetail.updateCell(row.NRSEQUENCIAL, 'CDCODIGO', newCodeDescription);
  }
  
  public beforeDeleteDetail({ event }: IEventParam<TekGrid>) {
    event.preventDefault();
    const utilities = new Utilities();
    const gridDetail = Metadata.getInstance<TekGrid>('detailsOperationalItems');
    const selectedRows = gridDetail.selectedRows;
    LoadingService.show();
    Http.post('/deleteDetail', {
      requestType: 'DataSet',
      dataset: selectedRows,
    }).then(async () => {
      this.beforeReloadDetail();
      utilities.successMessage();
    }).catch((e) => {
      utilities.errorMessage();
      console.warn(e);
    }).finally(() => {
      LoadingService.hide();
    });
  }
  
  public updateDetail({ component, event }: IEventParam<TekGrid>) {
    event.preventDefault();
    const gridOpItems = Metadata.getInstance<TekGrid>('gridOperationalItems');
    const utilities = new Utilities();
    const editedRows = component.getEditedRows();
    const currentRowGridItems = gridOpItems.datasource.currentRow;
    const data = [editedRows, currentRowGridItems];
    LoadingService.show();
    Http.post('/updateDetail', {
      requestType: 'DataSet',
      dataset: data,
    }).then(async () => {
      await component.saveEditedRows();
      // this.onInlineEditUpdateRow({ component });
      utilities.successMessage();
    }).catch((e) => {
      utilities.errorMessage();
      console.warn(e);
    }).finally(() => {
      this.gridDetailInEditing = false;
      LoadingService.hide();
    });
  }
}
