import {
  Modal,
  ModalService,
} from '@zeedhi/common';
import {
  Metadata,
  IDictionary,
} from '@zeedhi/core';
import {
  TekGrid,
} from '@zeedhi/teknisa-components-common';

export class DetailsBudgetApprovalModalController {
  private detailsBudgetApprovalModal!: Modal;
  
  public currentRowDetail = {} as any;
  
  public budgetDetails;
  
  public destroyDetailsBudgetApprovalModal() {
    this.detailsBudgetApprovalModal?.destroy();
  }
  
  public rowStyleGridConditions(row: IDictionary) {
    return {
      cssStyle: { background: row.VRORCAMENTOHIST !== row.VRORCAMENTO ? '#F4B800' : '' },
    };
  }
  
  public async openDetailsBudgetApprovalModal({ row }) {
    this.currentRowDetail = row;
    this.detailsBudgetApprovalModal = await ModalService.createFromJson('/modals/detailsBudgetApprovalModal', true);
    await this.detailsBudgetApprovalModal.show();
    await this.initializeDetailsBudget();
  }
  
  get setRouteDetailModal() {
    const currentRow = this.currentRowDetail;
    const route = '/crudN/detailsApprovalBudgetGop';
    const params = {
      IDTIPOORCAOP: currentRow.IDTIPOORCAOP,
      NRORCAMENTOGOP: currentRow.NRORCAMENTOGOP,
    };
    return {
      route,
      uniqueKey: 'SEQ',
      type: 'rest',
      limit: 10,
      lazyLoad: false,
      filter: params,
    };
  }
  
  private async initializeDetailsBudget() {
    const componentGridDetailsBudget = Metadata.getInstance<TekGrid>('gridDetailsBudgetApproval');
    const cdname: string = this.currentRowDetail.CDNMFILIAL;

    const items: string = this.currentRowDetail.ITENS;
    const title = `${cdname} - ${items}`;
    componentGridDetailsBudget.title = title;
  }
  
  public closeDetailsBudgetApprovalModal() {
    this.detailsBudgetApprovalModal.hide();
  }
}
